@@ -1,21 +1,37 @@
 /* ------------------------------------------------------------------------- */
 /* i2c-pcf-epp.c i2c-hw access for PCF8584 style EPP parallel port adapters  */
 /* ------------------------------------------------------------------------- */
+/*   Copyright (C) 1998-99 Hans Berglund
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.		     */
+/* ------------------------------------------------------------------------- */
+
+/* With some changes from Ryosuke Tajima <rosk@jsk.t.u-tokyo.ac.jp> */
 
 #include <linux/kernel.h>
 #include <linux/ioport.h>
 #include <linux/module.h>
 #include <linux/delay.h>
 #include <linux/slab.h>
-#include <linux/version.h>
 #include <linux/init.h>
 #include <linux/parport.h>
+#include "i2c.h"
+#include "i2c-algo-pcf.h"
 #include <asm/irq.h>
 #include <asm/io.h>
 
-#include "i2c.h"
-#include "i2c-algo-pcf.h"
-#include "i2c-pcf8584.h"
 
 struct  i2c_pcf_epp {
   int pe_base;
@@ -35,12 +51,9 @@ static int clock = 0;
 static int own   = 0;
 static int i2c_debug=0;
 static struct i2c_pcf_epp gpe;
-#if (LINUX_VERSION_CODE < 0x020301)
-static struct wait_queue *pcf_wait = NULL;
-#else
 static wait_queue_head_t pcf_wait;
-#endif
 static int pcf_pending;
+static spinlock_t irq_driver_lock = SPIN_LOCK_UNLOCKED;
 
 /* ----- global defines -----------------------------------------------	*/
 #define DEB(x)	if (i2c_debug>=1) x
@@ -50,7 +63,7 @@ static int pcf_pending;
 
 /* --- Convenience defines for the EPP/SPP port:			*/
 #define BASE	((struct i2c_pcf_epp *)(data))->pe_base
-#define DATA	BASE			/* SPP data port */
+// #define DATA	BASE			/* SPP data port */
 #define STAT	(BASE+1)		/* SPP status port */
 #define CTRL	(BASE+2)		/* SPP control port */
 #define EADD	(BASE+3)		/* EPP address port */
@@ -127,14 +140,14 @@ static void pcf_epp_waitforpin(void) {
   int timeout = 10;
 
   if (gpe.pe_irq > 0) {
-    cli();
+    spin_lock_irq(&irq_driver_lock);
     if (pcf_pending == 0) {
       interruptible_sleep_on_timeout(&pcf_wait, timeout*HZ);
       //udelay(100);
     } else {
       pcf_pending = 0;
     }
-    sti();
+    spin_unlock_irq(&irq_driver_lock);
   } else {
     udelay(100);
   }
@@ -177,69 +190,29 @@ static int pcf_epp_init(void *data)
   return 0;
 }
 
-
-static void pcf_epp_exit(void)
-{
-  if (gpe.pe_irq > 0) {
-    disable_irq(gpe.pe_irq);
-    free_irq(gpe.pe_irq, 0);
-  }
-  release_region(gpe.pe_base , 5);
-}
-
-
-static int pcf_epp_reg(struct i2c_client *client)
-{
-  return 0;
-}
-
-
-static int pcf_epp_unreg(struct i2c_client *client)
-{
-  return 0;
-}
-
-static void pcf_epp_inc_use(struct i2c_adapter *adap)
-{
-#ifdef MODULE
-  MOD_INC_USE_COUNT;
-#endif
-}
-
-static void pcf_epp_dec_use(struct i2c_adapter *adap)
-{
-#ifdef MODULE
-  MOD_DEC_USE_COUNT;
-#endif
-}
-
-
 /* ------------------------------------------------------------------------
  * Encapsulate the above functions in the correct operations structure.
  * This is only done when more than one hardware adapter is supported.
  */
 static struct i2c_algo_pcf_data pcf_epp_data = {
-  NULL,
-  pcf_epp_setbyte,
-  pcf_epp_getbyte,
-  pcf_epp_getown,
-  pcf_epp_getclock,
-  pcf_epp_waitforpin,
-  80, 80, 100,		/*	waits, timeout */
+	.setpcf	    = pcf_epp_setbyte,
+	.getpcf	    = pcf_epp_getbyte,
+	.getown	    = pcf_epp_getown,
+	.getclock   = pcf_epp_getclock,
+	.waitforpin = pcf_epp_waitforpin,
+	.udelay	    = 80,
+	.mdelay	    = 80,
+	.timeout    = HZ,
 };
 
 static struct i2c_adapter pcf_epp_ops = {
-  "PCF8584 EPP adapter",
-  I2C_HW_P_LP,
-  NULL,
-  &pcf_epp_data,
-  pcf_epp_inc_use,
-  pcf_epp_dec_use,
-  pcf_epp_reg,
-  pcf_epp_unreg,
+	.owner		= THIS_MODULE,
+	.name		= "PCF8584 EPP adapter",
+	.id		= I2C_HW_P_LP,
+	.algo_data	= &pcf_epp_data,
 };
 
-int __init i2c_pcfepp_init(void) 
+static int __init i2c_pcfepp_init(void) 
 {
   struct i2c_pcf_epp *pepp = &gpe;
 
@@ -268,9 +241,7 @@ int __init i2c_pcfepp_init(void)
     pepp->pe_own = own;
 
   pcf_epp_data.data = (void *)pepp;
-#if (LINUX_VERSION_CODE >= 0x020301)
   init_waitqueue_head(&pcf_wait);
-#endif
   if (pcf_epp_init(pepp) == 0) {
     int ret;
     if ( (ret = i2c_pcf_add_bus(&pcf_epp_ops)) < 0) {
@@ -286,12 +257,19 @@ int __init i2c_pcfepp_init(void)
   return 0;
 }
 
+static void __exit pcf_epp_exit(void)
+{
+  i2c_pcf_del_bus(&pcf_epp_ops);
+  if (gpe.pe_irq > 0) {
+    disable_irq(gpe.pe_irq);
+    free_irq(gpe.pe_irq, 0);
+  }
+  release_region(gpe.pe_base , 5);
+}
 
-EXPORT_NO_SYMBOLS;
-
-#ifdef MODULE
 MODULE_AUTHOR("Hans Berglund <hb@spacetec.no> \n modified by Ryosuke Tajima <rosk@jsk.t.u-tokyo.ac.jp>");
 MODULE_DESCRIPTION("I2C-Bus adapter routines for PCF8584 EPP parallel port adapter");
+MODULE_LICENSE("GPL");
 
 MODULE_PARM(base, "i");
 MODULE_PARM(irq, "i");
@@ -299,17 +277,5 @@ MODULE_PARM(clock, "i");
 MODULE_PARM(own, "i");
 MODULE_PARM(i2c_debug, "i");
 
-int init_module(void) 
-{
-  return i2c_pcfepp_init();
-}
-
-void cleanup_module(void) 
-{
-  i2c_pcf_del_bus(&pcf_epp_ops);
-  pcf_epp_exit();
-}
-
-#endif
-
-
+module_init(i2c_pcfepp_init);
+module_exit(pcf_epp_exit);
